package resources

import (
	"cloud.o-forge.io/core/oc-lib/models/common/pricing"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
)

type ResourceInterface interface {
	utils.DBObject
	Trim()
	ConvertToPricedResource(t tools.DataType, request *tools.APIRequest) pricing.PricedItemITF
	GetType() string
	GetSelectedInstance() utils.DBObject
	SetAllowedInstances(request *tools.APIRequest)
}

type ResourceInstanceITF interface {
	utils.DBObject
	GetID() string
	GetName() string
	StoreDraftDefault()
	GetPricingsProfiles(peerID string, groups []string) []pricing.PricingProfileITF
	GetPeerGroups() ([]ResourcePartnerITF, []map[string][]string)
	ClearPeerGroups()
}

type ResourcePartnerITF interface {
	GetPricingsProfiles(peerID string, groups []string) []pricing.PricingProfileITF
	GetPeerGroups() map[string][]string
	ClearPeerGroups()
}
